@@ -24,11 +24,11 @@
 from .config import VERSION, Var
 from .config import hosts_are_safe, Path
 from .filters import subscribe, subscribe_custom, update, update_all, rename_custom, sync_filter_list_repo, \
-    get_all_filter_lists, unsubscribe, change_permissions, Filter, purge_cache
-from .rules import allow_domains, block_domains, redirect_domains, delete_rules, Rule
+    get_all_filter_lists, unsubscribe, change_permissions, Filter, purge_cache, get_active_filter_lists_count
+from .rules import allow_domains, block_domains, redirect_domains, delete_rules, Rule, get_all_rules, get_rules_count
 from .const import RulePolicy, USER_RULE_PRIORITY, FilterPermissions, FilterSyntax
-from .hosts import update_hosts, restore_hosts
-from .daemon import start_daemon
+from .hosts import update_hosts, restore_hosts, enable_protection
+from tblock.daemon import start_daemon
 
 # This step is required. Otherwise, if using:
 # from .config import VERSION as __version__
