@@ -26,13 +26,13 @@ import sqlite3
 import sys
 
 # Local modules
-from .config import Var, log_message, create_dirs, VERSION
+from .config import Var, log_message, create_dirs, VERSION, Path
 from .filters import subscribe, subscribe_custom, update_all, change_permissions, unsubscribe, rename_custom, \
     sync_filter_list_repo, show_info, FilterPermissions, list_filter_lists, search_filter_lists, purge_cache, \
     query_domain_to_filter_list
-from .hosts import update_hosts, restore_hosts, gen_hosts
+from .hosts import update_hosts, restore_hosts, gen_hosts, enable_protection
 from .style import show_status
-from .daemon import start_daemon
+from tblock.daemon import start_daemon
 from .argumentor import Arguments
 from .rules import allow_domains, block_domains, redirect_domains, delete_rules, list_rules
 from .exceptions import RuleError, TBlockNetworkError, HostsError, FilterError, FilterSyntaxError, RepoError, \
@@ -206,17 +206,24 @@ def __setup_args_tblock(args: list) -> Arguments:
 
     # Add "Hosts" operations
     parser.add_operation(
-        operation="--restore",
+        operation="--disable",
         short_operation="-D",
         value_type=bool,
         description="Restore default hosts file",
         group="Hosts"
     )
+    parser.add_operation(
+        operation="--enable",
+        short_operation="-E",
+        value_type=bool,
+        description="Enable protection without rebuild hosts file",
+        group="Hosts"
+    )
     parser.add_operation(
         operation="--update-hosts",
         short_operation="-H",
         value_type=bool,
-        description="Update hosts file",
+        description="Rebuild hosts file",
         group="Hosts"
     )
     parser.add_operation(
@@ -530,9 +537,9 @@ def __setup_args_tblockd(args: list) -> Arguments:
         option="--config",
         short_option="-c",
         value_type=str,
-        required_by_operations=["--daemon"],
         available_with_operations=["--daemon"],
-        description="Path to the config file to use"
+        description="Path to the config file to use",
+        default_value=Path.CONFIG
     )
     parser.add_option(
         option="--no-pid",
@@ -660,11 +667,16 @@ def __parse_args_tblock(args: list) -> None:
             do_not_prompt=options["--no-prompt"],
             quiet=False
         )
-    elif operations["--restore"]:
+    elif operations["--disable"]:
         restore_hosts(
             quiet=False,
             do_not_prompt=options["--no-prompt"]
         )
+    elif operations["--enable"]:
+        enable_protection(
+            quiet=False,
+            do_not_prompt=options["--no-prompt"]
+        )
     elif operations["--update-hosts"]:
         update_hosts(
             quiet=False,
