@@ -32,7 +32,7 @@ import sqlite3
 from .const import FilterPermissions
 
 
-VERSION = "2.2.0"
+VERSION = "2.4.1"
 
 
 class Path:
@@ -77,6 +77,7 @@ class Path:
     DATABASE = os.path.join(PREFIX, 'storage.sqlite')
     HOSTS_BACKUP = os.path.join(PREFIX, 'hosts.bak')
     DB_LOCK = os.path.join(PREFIX, '.db_lock')
+    BUILT_HOSTS_BACKUP = os.path.join(PREFIX, 'active.hosts.bak')
 
 
 def load_config(config_file: str) -> configparser.ConfigParser:
@@ -92,7 +93,7 @@ class Var:
     try:
         DEFAULT_PERMISSIONS = FilterPermissions(__config.get("default", "default_permissions"))
     except (configparser.NoSectionError, configparser.NoOptionError, ValueError):
-        DEFAULT_PERMISSIONS = FilterPermissions('b')
+        DEFAULT_PERMISSIONS = FilterPermissions('AB')
     try:
         DEFAULT_IP = __config.get("default", "default_ip")
     except (configparser.NoSectionError, configparser.NoOptionError, ValueError):
@@ -106,12 +107,10 @@ class Var:
     except (configparser.NoSectionError, configparser.NoOptionError, ValueError):
         ALLOW_IPV6 = False
     REPO_MIRRORS = [
-        "https://tblock.codeberg.page/repo/index.json",
-        "https://codeberg.org/tblock/repo/raw/branch/pages/index.json",
-        "https://framagit.org/twann/repo/-/raw/pages/index.json",
-        "https://git.disroot.org/tblock/repo/raw/branch/pages/index.json",
-        "https://0xacab.org/twann/repo/-/raw/pages/index.json",
-        "http://wmj5kiic7b6kjplpbvwadnht2nh2qnkbnqtcv3dyvpqtz7ssbssftxid.onion/twann/repo/-/raw/pages/index.json"
+        "https://tblock.codeberg.page/repo/2.4.0/index.json",
+        "https://codeberg.org/tblock/repo/raw/branch/pages/2.4.0/index.json",
+        "https://framagit.org/twann/repo/-/raw/pages/2.4.0/index.json",
+        "https://git.disroot.org/tblock/repo/raw/branch/pages/2.4.0/index.json"
     ]
 
 
