@@ -35,13 +35,14 @@ from ..config import Path
 
 
 ALL_SYNTAX = [FilterSyntax.ADBLOCKPLUS, FilterSyntax.DNSMASQ, FilterSyntax.HOSTS, FilterSyntax.LIST,
-              FilterSyntax.TBLOCK]
+              FilterSyntax.TBLOCK, FilterSyntax.TBLOCK_LEGACY]
 
 ADBLOCKPLUS = FilterSyntax.ADBLOCKPLUS
 DNSMASQ = FilterSyntax.DNSMASQ
 HOSTS = FilterSyntax.HOSTS
 LIST = FilterSyntax.LIST
 TBLOCK = FilterSyntax.TBLOCK
+TBLOCK_LEGACY = FilterSyntax.TBLOCK_LEGACY
 UNKNOWN = FilterSyntax.UNKNOWN
 
 
@@ -80,6 +81,12 @@ class FilterParser:
             raise FilterSyntaxError("unable to access file: {0}".format(self.file))
         except FileNotFoundError:
             raise FilterSyntaxError("file not found: {0}".format(self.file))
+
+        if not self.quiet and self.syntax == FilterSyntax.TBLOCK_LEGACY:
+            print(
+                f" {Icon.WARNING} TBlock filter format v1 is deprecated. In future versions, only v2 will be supported."
+            )
+
         count = 0
         percent = 0
         all_rules_count = len(data)
@@ -101,10 +108,14 @@ class FilterParser:
                     loading_icon(count), all_rules_count, percent), end="\r")
 
             if is_valid_rule(line.split("\n")[0], self.syntax):
-                if self.syntax == FilterSyntax.TBLOCK:
+                if self.syntax == FilterSyntax.TBLOCK_LEGACY:
                     rule = get_rule(line.split("\n")[0], self.syntax, allow_comments=False, tblock_begin=tblock_begin,
                                     tblock_policy=tblock_policy, tblock_ip=tblock_ip, allow_allow=permissions.ALLOW,
                                     allow_block=permissions.BLOCK, allow_redirect=permissions.REDIRECT)
+                elif self.syntax == FilterSyntax.TBLOCK:
+                    rule = get_rule(line.split("\n")[0], self.syntax, allow_comments=False,
+                                    tblock_policy=tblock_policy, tblock_ip=tblock_ip, allow_allow=permissions.ALLOW,
+                                    allow_block=permissions.BLOCK, allow_redirect=permissions.REDIRECT)
                 else:
                     rule = get_rule(line.split("\n")[0], self.syntax, allow_comments=False,
                                     allow_allow=permissions.ALLOW, allow_block=permissions.BLOCK,
@@ -118,7 +129,7 @@ class FilterParser:
                         ip = None
 
                     # TBlock syntax support
-                    if self.syntax == FilterSyntax.TBLOCK:
+                    if self.syntax in (FilterSyntax.TBLOCK_LEGACY, FilterSyntax.TBLOCK):
                         if rule[0] == TBlockSyntaxStatus.CURRENT_STATUS:
                             tblock_begin = rule[1]
                             continue
@@ -176,6 +187,11 @@ class FilterParser:
         if not self.quiet:
             print(" {0} Input syntax is: {1}".format(Icon.INFORMATION, self.syntax))
 
+        if not self.quiet and FilterSyntax.TBLOCK_LEGACY in (self.syntax, output_syntax):
+            print(
+                f" {Icon.WARNING} TBlock filter format v1 is deprecated. In future versions, only v2 will be supported."
+            )
+
         # TBlock syntax support
         tblock_begin = False
         tblock_policy = None
@@ -189,15 +205,18 @@ class FilterParser:
         with open(output_file, "wt") as w:
             if output_syntax == FilterSyntax.ADBLOCKPLUS:
                 w.write("[Adblock Plus 2.0]\n\n")
-            elif output_syntax == FilterSyntax.TBLOCK:
+            elif output_syntax == FilterSyntax.TBLOCK_LEGACY:
                 w.write("@BEGIN_RULES\n\n")
 
             for line in data:
 
                 if is_valid_rule(line.split("\n")[0], self.syntax):
-                    if self.syntax == FilterSyntax.TBLOCK:
+                    if self.syntax == FilterSyntax.TBLOCK_LEGACY:
                         rule = get_rule(line.split("\n")[0], self.syntax, allow_comments=allow_comments,
                                         tblock_begin=tblock_begin, tblock_policy=tblock_policy, tblock_ip=tblock_ip)
+                    elif self.syntax == FilterSyntax.TBLOCK:
+                        rule = get_rule(line.split("\n")[0], self.syntax, allow_comments=allow_comments,
+                                        tblock_policy=tblock_policy, tblock_ip=tblock_ip)
                     else:
                         rule = get_rule(line.split("\n")[0], self.syntax, allow_comments=allow_comments)
 
@@ -209,7 +228,7 @@ class FilterParser:
                             ip = None
 
                         # TBlock syntax support
-                        if self.syntax == FilterSyntax.TBLOCK:
+                        if self.syntax in (FilterSyntax.TBLOCK_LEGACY, FilterSyntax.TBLOCK):
                             if rule[0] == TBlockSyntaxStatus.CURRENT_STATUS:
                                 tblock_begin = rule[1]
                                 continue
@@ -264,6 +283,22 @@ class FilterParser:
                                 w.write(f"#{rule[0]}\n")
 
                         elif output_syntax == FilterSyntax.TBLOCK:
+                            if not rule[1] == output_tblock_policy or not ip == output_tblock_ip:
+                                if rule[1] == RulePolicy.ALLOW:
+                                    w.write("\n[allow]\n")
+                                elif rule[1] == RulePolicy.BLOCK and not contains_wildcards(rule[0]):
+                                    w.write("\n[block]\n")
+                                elif rule[1] == RulePolicy.REDIRECT and not contains_wildcards(rule[0]):
+                                    w.write(f'\n[redirect "{ip}"]\n')
+                                if rule[1] != RULE_COMMENTS:
+                                    output_tblock_policy = rule[1]
+                                    output_tblock_ip = ip
+                            if not rule[1] == RULE_COMMENTS:
+                                w.write(f"{rule[0]}\n")
+                            elif allow_comments:
+                                w.write(f"#{rule[0]}\n")
+
+                        elif output_syntax == FilterSyntax.TBLOCK_LEGACY:
                             if not rule[1] == output_tblock_policy or not ip == output_tblock_ip:
                                 if rule[1] == RulePolicy.ALLOW:
                                     w.write("\n!allow\n")
@@ -286,7 +321,7 @@ class FilterParser:
                 elif line == "\n" and not optimize:
                     w.write("\n")
 
-            if output_syntax == FilterSyntax.TBLOCK:
+            if output_syntax == FilterSyntax.TBLOCK_LEGACY:
                 w.write("\n\n@END_RULES")
 
         if not self.quiet:
