@@ -30,9 +30,10 @@ import json
 import hashlib
 import lzma
 import gzip
+from colorama import Fore, Style
 
 # Local modules
-from .const import FilterPermissions, WARNS, RulePolicy
+from .const import FilterPermissions, WARNS, RulePolicy, USER_RULE_PRIORITY
 from .style import Icon, Font, loading_icon
 from .config import Path, Var
 from .utils import fetch_file, prompt_user, is_url, check_root_access, lock_db, unlock_db
@@ -91,11 +92,11 @@ class Filter:
         self.permissions = FilterPermissions(data[4]) if self.exists else None
         self.mirrors = json.loads(data[5]) if (self.exists and data[5]) else {}
         self.syntax = data[6] if self.exists else None
-        self.rules_count = data[7] if self.exists else None
+        self.rules_count = data[7] if self.exists else 0
         self.tmp_file = os.path.join(Path.TMP_DIR, self.id.replace(":", "_").replace("/", "_").replace("?", "_"))
         self.cache_file = os.path.join(Path.CACHE, self.id.replace(":", "_").replace("/", "_").replace("?", "_"))
 
-    def __retrieve_mirror(self, mirror: str, compression: str = None) -> bool:
+    def retrieve_mirror(self, mirror: str, compression: str = None) -> bool:
         """
         Retrieve a mirror of a filter list when the main location is unreachable
 
@@ -110,41 +111,47 @@ class Filter:
             elif compression == "xz":
                 # Decompress XZ-compressed filter list
                 __msg = "Decompressing filter list: {0} (xz)".format(self.id)
-                print(f" {loading_icon(1)} {__msg}", end='\r')
+                if not self.quiet:
+                    print(f" {loading_icon(1)} {__msg}", end='\r')
                 with open(self.tmp_file, "rb") as f:
                     filter_list_content = lzma.decompress(f.read(), format=lzma.FORMAT_XZ)
-                print(f" {loading_icon(2)} {__msg}", end='\r')
+                if not self.quiet:
+                    print(f" {loading_icon(2)} {__msg}", end='\r')
                 with open(self.tmp_file, "wb") as w:
                     w.write(filter_list_content)
-                print(f" {Icon.SUCCESS} {__msg}")
+                if not self.quiet:
+                    print(f" {Icon.SUCCESS} {__msg}")
                 return True
             elif compression == "gzip":
                 # Decompress gzip-compressed filter list
                 __msg = "Decompressing filter list: {0} (gzip)".format(self.id)
-                print(f" {loading_icon(1)} {__msg}", end='\r')
+                if not self.quiet:
+                    print(f" {loading_icon(1)} {__msg}", end='\r')
                 with open(self.tmp_file, "rb") as f:
                     filter_list_content = gzip.decompress(f.read())
-                print(f" {loading_icon(2)} {__msg}", end='\r')
+                if not self.quiet:
+                    print(f" {loading_icon(2)} {__msg}", end='\r')
                 with open(self.tmp_file, "wb") as w:
                     w.write(filter_list_content)
-                print(f" {Icon.SUCCESS} {__msg}")
+                if not self.quiet:
+                    print(f" {Icon.SUCCESS} {__msg}")
                 return True
             else:
                 raise FilterError("compression algorithm is currently unsupported: {0}".format(compression))
         else:
             return False
 
-    def __cache_exists(self) -> bool:
+    def cache_exists(self) -> bool:
         """
         Check if the filter list already exists in cache
         """
         return os.path.exists(self.cache_file)
 
-    def __cache_is_up_to_date(self) -> bool:
+    def cache_is_up_to_date(self) -> bool:
         """
         Compare the downloaded filter list with the one stored in cache and check if filter list is already up-to-date
         """
-        if self.__cache_exists():
+        if self.cache_exists():
             with open(self.tmp_file, 'rb') as f:
                 tmp_md5sum = hashlib.sha512(f.read())
             with open(self.cache_file, 'rb') as f:
@@ -169,9 +176,11 @@ class Filter:
         Delete the filter list stored in cache
         """
         __msg = "Removing cache: {0}".format(self.id)
-        print(f" {loading_icon(1)} {__msg}", end='\r')
-        if self.__cache_exists():
-            print(f" {Icon.SUCCESS} {__msg}")
+        if not self.quiet:
+            print(f" {loading_icon(1)} {__msg}", end='\r')
+        if self.cache_exists():
+            if not self.quiet:
+                print(f" {Icon.SUCCESS} {__msg}")
             os.remove(self.cache_file)
             try:
                 with sqlite3.connect(Path.DATABASE) as conn:
@@ -181,7 +190,7 @@ class Filter:
                     )
             except sqlite3.OperationalError:
                 pass
-        else:
+        elif not self.quiet:
             print(f" {Icon.WARNING} Cache does not exist: {self.id}")
 
     def retrieve(self) -> None:
@@ -194,10 +203,10 @@ class Filter:
             if not fetch_file(self.source, self.id, self.tmp_file, self.quiet):
                 for mirror in self.mirrors.keys():
                     if "compression" in self.mirrors[mirror].keys():
-                        if self.__retrieve_mirror(mirror, compression=self.mirrors[mirror]["compression"]):
+                        if self.retrieve_mirror(mirror, compression=self.mirrors[mirror]["compression"]):
                             break
                     else:
-                        if self.__retrieve_mirror(mirror):
+                        if self.retrieve_mirror(mirror):
                             break
                 else:
                     try:
@@ -221,7 +230,7 @@ class Filter:
         """
         if not os.path.isfile(self.tmp_file):
             raise FilterError("filter list needs to be downloaded before updating its rules")
-        elif self.__cache_is_up_to_date() and self.get_rules_count() == self.rules_count and not force:
+        elif self.cache_is_up_to_date() and self.get_rules_count() == self.rules_count and not force:
             print(f" {Icon.SUCCESS} " + "Filter list is up-to-date")
         else:
 
@@ -346,6 +355,10 @@ class Filter:
             raise FilterError("filter list source already exists: {0}".format(self.source))
         elif custom_syntax is not None and custom_syntax not in ALL_SYNTAX:
             raise FilterSyntaxError("invalid syntax: {0}".format(custom_syntax))
+        elif self.id == USER_RULE_PRIORITY:
+            raise FilterError("filter list cannot have the same ID than user rules: {0}".format(self.id))
+        elif not self.source:
+            raise FilterError("invalid source: {0}".format(self.source))
         else:
             # Get the real path of the file if it is a local file
             if not is_url(self.source):
@@ -360,6 +373,8 @@ class Filter:
                     "VALUES (?, ?, ?, ?, ?, ?, ?, ?);",
                     (self.id, self.source, json.dumps({}), int(False), int(False), None, json.dumps({}), custom_syntax)
                 )
+            self.exists = True
+            self.syntax = custom_syntax
             if not self.quiet:
                 print(f" {Icon.SUCCESS} {__msg} {self.id}")
 
@@ -381,11 +396,12 @@ class Filter:
                         "UPDATE filters SET subscribing=? WHERE id=?",
                         (int(False), self.id)
                     )
+                self.__init__(self.id, quiet=self.quiet)
                 if not self.quiet:
                     print(f" {Icon.SUCCESS} {__msg}")
             else:
                 __msg = "Removing filter list from database"
-                if not self.subscribing:
+                if not self.subscribing and not self.quiet:
                     print(f" {Icon.WARNING} " + "Not subscribing to custom filter list")
                 if not self.quiet:
                     print(f" {loading_icon(1)} {__msg}", end='\r')
@@ -394,6 +410,7 @@ class Filter:
                         "DELETE FROM filters WHERE id=?",
                         (self.id,)
                     )
+                self.__init__(self.id, quiet=self.quiet, custom_source=self.source)
                 if not self.quiet:
                     print(f" {Icon.SUCCESS} {__msg}")
             # self.delete_cache()
@@ -434,6 +451,8 @@ class Filter:
                 print(f" {Icon.WARNING} Not subscribing to filter list")
         elif Filter(filter_id).exists:
             raise FilterError("filter list ID already exists: {0}".format(filter_id))
+        elif filter_id == USER_RULE_PRIORITY:
+            raise FilterError("filter list cannot have the same ID than user rules: {0}".format(self.id))
         else:
             __msg = "Changing ID of filter list:"
             if not self.quiet:
@@ -541,23 +560,23 @@ def get_all_filter_lists(subscribing_only: bool = False, not_subscribing_only: b
     else:
         with sqlite3.connect(Path.DATABASE) as conn:
             if subscribing_only and from_repo_only:
-                data = conn.execute("SELECT id FROM filters WHERE subscribing=1 AND on_repo=1;")
+                data = conn.execute("SELECT id FROM filters WHERE subscribing=1 AND on_repo=1 ORDER BY id ASC;")
             elif subscribing_only and custom_only:
-                data = conn.execute("SELECT id FROM filters WHERE subscribing=1 AND on_repo=0;")
+                data = conn.execute("SELECT id FROM filters WHERE subscribing=1 AND on_repo=0 ORDER BY id ASC;")
             elif subscribing_only:
-                data = conn.execute("SELECT id FROM filters WHERE subscribing=1;")
+                data = conn.execute("SELECT id FROM filters WHERE subscribing=1 ORDER BY id ASC;")
             elif not_subscribing_only and from_repo_only:
-                data = conn.execute("SELECT id FROM filters WHERE subscribing=0 AND on_repo=1;")
+                data = conn.execute("SELECT id FROM filters WHERE subscribing=0 AND on_repo=1 ORDER BY id ASC;")
             elif not_subscribing_only and custom_only:
-                data = conn.execute("SELECT id FROM filters WHERE subscribing=0 AND on_repo=0;")
+                data = conn.execute("SELECT id FROM filters WHERE subscribing=0 AND on_repo=0 ORDER BY id ASC;")
             elif not_subscribing_only:
-                data = conn.execute("SELECT id FROM filters WHERE subscribing=0;")
+                data = conn.execute("SELECT id FROM filters WHERE subscribing=0 ORDER BY id ASC;")
             elif custom_only:
-                data = conn.execute("SELECT id FROM filters WHERE on_repo=0;")
+                data = conn.execute("SELECT id FROM filters WHERE on_repo=0 ORDER BY id ASC;")
             elif from_repo_only:
-                data = conn.execute("SELECT id FROM filters WHERE on_repo=1;")
+                data = conn.execute("SELECT id FROM filters WHERE on_repo=1 ORDER BY id ASC;")
             else:
-                data = conn.execute("SELECT id FROM filters;")
+                data = conn.execute("SELECT id FROM filters ORDER BY id ASC;")
         output_list = []
         for row in data:
             if blacklist and row[0] in blacklist:
@@ -616,6 +635,8 @@ def subscribe(filter_lists: list, do_not_prompt: bool = False, quiet: bool = Fal
 
     # Check all filter lists
     if not sync_repo:
+        if get_current_repo_index_version() == 0 and not quiet:
+            print(f"{Fore.YELLOW}Warning: repository index is not synced. Run tblock -Y to sync now.{Style.RESET_ALL}")
         if __check_filter_lists_validity_to_subscribe(filter_lists):
             # Prompt the user before continuing
             if do_not_prompt or prompt_user("You are about to subscribe to the following filters:", filter_lists):
@@ -1101,6 +1122,21 @@ def query_domain_to_filter_list(domains: list, quiet: bool = False) -> None:
                 print(f"{origin}: REDIRECT {d} TO {domain_obj.ip}")
 
 
+def get_current_repo_index_version() -> int:
+    with sqlite3.connect(Path.DATABASE) as conn:
+        try:
+            return int(conn.cursor().execute(
+                'SELECT value FROM system WHERE variable="repo_version";'
+            ).fetchone()[0])
+        except (IndexError, TypeError):
+            return 0
+
+
+def get_active_filter_lists_count() -> int:
+    with sqlite3.connect(Path.DATABASE) as db:
+        return db.cursor().execute('SELECT COUNT() FROM filters WHERE subscribing=1;').fetchone()[0]
+
+
 def sync_filter_list_repo(quiet: bool = False, force: bool = False, lock_database: bool = True) -> None:
     """
     Print information about a given list of filter lists
@@ -1123,9 +1159,9 @@ def sync_filter_list_repo(quiet: bool = False, force: bool = False, lock_databas
 
     for mirror in Var.REPO_MIRRORS:
         if fetch_file(mirror, "filter list repository",
-                      os.path.join(Path.TMP_DIR, "index.json")) and \
+                      os.path.join(Path.TMP_DIR, "index.json"), quiet=quiet) and \
            fetch_file(mirror + ".sha512", "filter list repository checksum",
-                      os.path.join(Path.TMP_DIR, "index.json.sha512")):
+                      os.path.join(Path.TMP_DIR, "index.json.sha512"), quiet=quiet):
             break
     else:
         raise RepoError("unable to retrieve filter list repository")
@@ -1146,7 +1182,8 @@ def sync_filter_list_repo(quiet: bool = False, force: bool = False, lock_databas
         if not quiet:
             print(f" {Icon.SUCCESS} {__msg}")
     else:
-        print(f" {Icon.ERROR} {__msg}")
+        if not quiet:
+            print(f" {Icon.ERROR} {__msg}")
         raise RepoError("wrong checksum for filter list repository")
 
     # Parse the JSON file
@@ -1166,15 +1203,11 @@ def sync_filter_list_repo(quiet: bool = False, force: bool = False, lock_databas
             print(f" {Icon.ERROR} {__msg}")
         raise RepoError("filter list index is not a valid JSON file")
 
+    new_repo_version = data["repo"]["version"]
+    current_repo_version = get_current_repo_index_version()
+
     db = sqlite3.connect(Path.DATABASE)
     cursor = db.cursor()
-    new_repo_version = data["repo"]["version"]
-    try:
-        current_repo_version = int(cursor.execute(
-            'SELECT value FROM system WHERE variable="repo_version";'
-        ).fetchone()[0])
-    except (IndexError, TypeError):
-        current_repo_version = 0
 
     # Check if index is up-to-date
     if len(str(current_repo_version)) >= 6 and int(str(current_repo_version)[0:6]) >= int(data["repo"]["version"]) and \
@@ -1185,7 +1218,8 @@ def sync_filter_list_repo(quiet: bool = False, force: bool = False, lock_databas
         db.close()
         unlock_db()
     else:
-        print(f" {Icon.SUCCESS} {__msg}")
+        if not quiet:
+            print(f" {Icon.SUCCESS} {__msg}")
 
         # Check if TBlock is compatible and if it has been updated upstream
         __msg = "Checking compatibility"
@@ -1193,19 +1227,23 @@ def sync_filter_list_repo(quiet: bool = False, force: bool = False, lock_databas
             min_tblock_compat = int(data["repo"]["min_compat"].replace(".", ""))
             tblock_latest = str(data["repo"]["latest_version"])
         except KeyError:
-            print(f" {Icon.ERROR} {__msg}")
-            print(f" {Icon.WARNING} " + "Repository does not seem to indicate a compatible version")
+            if not quiet:
+                print(f" {Icon.ERROR} {__msg}")
+                print(f" {Icon.WARNING} " + "Repository does not seem to indicate a compatible version")
         else:
             if int(TBLOCK_VERSION.replace(".", "").split("-")[0]) < min_tblock_compat:
-                print(f" {Icon.ERROR} {__msg}")
+                if not quiet:
+                    print(f" {Icon.ERROR} {__msg}")
                 raise RepoError("index is not compatible with current tblock version")
-            else:
+            elif not quiet:
                 print(f" {Icon.SUCCESS} {__msg}")
             if int(TBLOCK_VERSION.replace(".", "").split("-")[0]) < int(tblock_latest.replace(".", "")):
-                print(f" {Icon.INFORMATION} " + "A new version of TBlock is available: {0}".format(tblock_latest))
+                if not quiet:
+                    print(f" {Icon.INFORMATION} " + "A new version of TBlock is available: {0}".format(tblock_latest))
 
-        print(f" {Icon.INFORMATION} " + "Upgrading from version {0} to {1}".format(
-            current_repo_version, new_repo_version))
+        if not quiet:
+            print(f" {Icon.INFORMATION} " + "Upgrading from version {0} to {1}".format(
+                current_repo_version, new_repo_version))
 
         # Upgrade the repository
         total_filter_lists = len(data["filters"])
@@ -1268,8 +1306,9 @@ def sync_filter_list_repo(quiet: bool = False, force: bool = False, lock_databas
                         (source, json.dumps(metadata), json.dumps(mirrors), syntax, filter_obj.id)
                     )
                 except sqlite3.IntegrityError:
-                    print(f" {Icon.WARNING} Upstream filter list failed to sync because its source already exists: "
-                          f"{_filter_list}")
+                    if not quiet:
+                        print(f" {Icon.WARNING} Upstream filter list failed to sync because its source already exists: "
+                              f"{_filter_list}")
 
             # If you are reading this, peace and love to you.
             # Be who you are and who you want to be. You are perfect.
@@ -1285,10 +1324,12 @@ def sync_filter_list_repo(quiet: bool = False, force: bool = False, lock_databas
                 except sqlite3.IntegrityError:
                     # That means the source is already taken
                     conflict_id = cursor.execute("SELECT id FROM filters WHERE source=?", (source,)).fetchone()[0]
-                    print(f" {Icon.WARNING} Local filter list source conflicts with upstream: {conflict_id}")
+                    if not quiet:
+                        print(f" {Icon.WARNING} Local filter list source conflicts with upstream: {conflict_id}")
 
             elif filter_obj.exists and not filter_obj.on_repo:
-                print(f" {Icon.WARNING} Local filter list conflicts with upstream: {filter_obj.id}")
+                if not quiet:
+                    print(f" {Icon.WARNING} Local filter list conflicts with upstream: {filter_obj.id}")
 
         # Check for all filter lists that were removed from the repository
         for filter_list in get_all_filter_lists(from_repo_only=True, subscribing_only=False):
@@ -1321,7 +1362,8 @@ def sync_filter_list_repo(quiet: bool = False, force: bool = False, lock_databas
 
         db.commit()
         db.close()
-        print(f" {Icon.SUCCESS} {__msg} {count}/{total_filter_lists} (100%)")
+        if not quiet:
+            print(f" {Icon.SUCCESS} {__msg} {count}/{total_filter_lists} (100%)")
 
         # Unlock the database
         if lock_database:
