@@ -57,6 +57,56 @@ def get_hostname() -> str:
             return ''
 
 
+def enable_protection(quiet: bool = False, do_not_prompt: bool = True) -> None:
+    if not check_root_access():
+        raise PermissionError("you need to run as root to perform this operation")
+
+    if do_not_prompt or prompt_user("You are about to replace your default hosts file with the one built by TBlock"):
+
+        if not quiet:
+            print(f"{Font.BOLD}==> Applying built hosts file")
+
+        if hosts_are_default():
+            if os.path.isfile(Path.BUILT_HOSTS_BACKUP):
+                if not quiet:
+                    print(f" {loading_icon(1)} Checking checksum", end="\r")
+                with sqlite3.connect(Path.DATABASE) as conn:
+                    s = conn.cursor().execute("SELECT value FROM system WHERE variable='hosts_shasum';",).fetchone()[0]
+                with open(os.path.join(Path.BUILT_HOSTS_BACKUP), "rb") as f:
+                    content = f.read()
+                    shasum = hashlib.sha512(content).hexdigest()
+                if s == shasum:
+                    if not quiet:
+                        print(f" {Icon.SUCCESS} Checking checksum")
+                    __msg = "Saving default hosts file"
+                    if not quiet:
+                        print(f" {loading_icon(1)} {__msg}", end="\r")
+                    with open(Path.HOSTS, "rt") as f:
+                        default_hosts = f.read()
+                    with open(Path.HOSTS_BACKUP, "wt") as w:
+                        w.write(default_hosts)
+                    if not quiet:
+                        print(f" {Icon.SUCCESS} {__msg}")
+                        print(f" {loading_icon(1)} Applying built hosts file", end="\r")
+                    with open(Path.HOSTS, "wb") as f:
+                        f.write(content)
+                    os.remove(Path.BUILT_HOSTS_BACKUP)
+                    if not quiet:
+                        print(f" {Icon.SUCCESS} Applying built hosts file")
+                        print(f" {Icon.INFORMATION} " + "Protection is now enabled")
+                else:
+                    if not quiet:
+                        print(f" {Icon.ERROR} Checking checksum")
+                        print(f" {Icon.INFORMATION} Run tblock -H to build the hosts file")
+                        raise SystemExit(1)
+            elif not quiet:
+                print(f" {Icon.WARNING} TBlock has never built the hosts file")
+                print(f" {Icon.INFORMATION} Run tblock -H to build the hosts file")
+        elif not quiet:
+            print(f" {Icon.WARNING} Protection is already active")
+            print(f" {Icon.INFORMATION} Run tblock -H to rebuild the hosts file")
+
+
 def update_hosts(quiet: bool = False, do_not_prompt: bool = True) -> None:
 
     # Check root access
@@ -90,7 +140,9 @@ def update_hosts(quiet: bool = False, do_not_prompt: bool = True) -> None:
             db = sqlite3.connect(Path.DATABASE)
             cursor = db.cursor()
             total_rules = cursor.execute('SELECT COUNT() FROM rules;').fetchone()[0]
-            rules = cursor.execute("SELECT domain, policy, ip FROM rules WHERE policy != ?;", (RulePolicy.ALLOW,))
+            rules = cursor.execute(
+                "SELECT domain, policy, ip FROM rules WHERE policy != ? ORDER BY domain ASC;", (RulePolicy.ALLOW,)
+            )
             count = 0
             invalid = False
             __msg = "Retrieving rules:"
@@ -121,7 +173,8 @@ def update_hosts(quiet: bool = False, do_not_prompt: bool = True) -> None:
 
         # Write changes into hosts file
         __msg = "Writing new hosts file"
-        print(f" {loading_icon(1)} {__msg}", end="\r")
+        if not quiet:
+            print(f" {loading_icon(1)} {__msg}", end="\r")
         with open(os.path.join(Path.TMP_DIR, "hosts.txt"), "rb") as f:
             content = f.read()
             shasum = hashlib.sha512(content).hexdigest()
@@ -132,7 +185,8 @@ def update_hosts(quiet: bool = False, do_not_prompt: bool = True) -> None:
             )
         with open(Path.HOSTS, "wb") as w:
             w.write(content)
-        print(f" {Icon.SUCCESS} {__msg}")
+        if not quiet:
+            print(f" {Icon.SUCCESS} {__msg}")
         if not quiet and enabling_protection:
             print(f" {Icon.INFORMATION} " + "Protection is now enabled")
 
@@ -148,15 +202,15 @@ def restore_hosts(quiet: bool = False, do_not_prompt: bool = True) -> None:
             __msg = "Restoring default hosts file"
             if not quiet:
                 print(f"{Font.BOLD}==> {__msg}")
+                print(f" {loading_icon(1)} Saving built hosts file", end="\r")
+            with open(Path.HOSTS, "rt") as f:
+                with open(Path.BUILT_HOSTS_BACKUP, "wt") as w:
+                    w.write(f.read())
+            if not quiet:
+                print(f" {Icon.SUCCESS} Saving built hosts file")
                 print(f" {loading_icon(1)} {__msg}", end="\r")
             with open(Path.HOSTS_BACKUP, "rt") as f:
                 content = f.read()
-                shasum = hashlib.sha512(content.encode("utf-8")).hexdigest()
-            with sqlite3.connect(Path.DATABASE) as conn:
-                conn.cursor().execute(
-                    "UPDATE system SET value=? WHERE variable='hosts_shasum';",
-                    (shasum,)
-                )
             with open(Path.HOSTS, "wt") as w:
                 w.write(content)
             os.remove(Path.HOSTS_BACKUP)
@@ -199,7 +253,7 @@ def remove_from_hosts(domains: dict, quiet: bool = False) -> None:
             f.write(new_hosts)
         if not quiet:
             print(f" {Icon.SUCCESS} Applying new rules to hosts file")
-    else:
+    elif not quiet:
         print(f" {Icon.ERROR} Applying new rules to hosts file")
         print(f" {Icon.WARNING} TBlock is not active")
 
@@ -234,7 +288,7 @@ def add_to_hosts(domains: list, ip: str = None, quiet: bool = False) -> None:
             f.write(content)
         if not quiet:
             print(f" {Icon.SUCCESS} Applying new rules to hosts file")
-    else:
+    elif not quiet:
         print(f" {Icon.ERROR} Applying new rules to hosts file")
         print(f" {Icon.WARNING} TBlock is not active")
 
