@@ -74,7 +74,7 @@ class Rule:
         :param ip: Optional. The bind address domains should be redirected to (only when the policy is redirect)
         :param quiet: Optional. Do not display an output (false by default)
         """
-        if self.exists and self.policy == policy and self.ip == ip:
+        if self.exists and self.policy == policy and self.ip == ip and self.filter_id == USER_RULE_PRIORITY:
             if not quiet:
                 print(f" {Icon.WARNING} Exact same rule already exists for domain: {self.domain}")
         elif not is_valid_domain(self.domain, allow_wildcards=bool(policy == RulePolicy.ALLOW)):
@@ -98,6 +98,7 @@ class Rule:
                                         (policy, ip_address.compressed, USER_RULE_PRIORITY, self.domain)
                                     )
                                     conn.commit()
+                                self.__init__(self.domain)
                             else:
                                 with sqlite3.connect(Path.DATABASE) as conn:
                                     conn.cursor().execute(
@@ -106,6 +107,7 @@ class Rule:
                                         (self.domain, policy, ip_address.compressed, USER_RULE_PRIORITY)
                                     )
                                     conn.commit()
+                                self.__init__(self.domain)
 
                         else:
                             if self.exists:
@@ -115,6 +117,7 @@ class Rule:
                                         (policy, None, USER_RULE_PRIORITY, self.domain)
                                     )
                                     conn.commit()
+                                self.__init__(self.domain)
                             else:
                                 with sqlite3.connect(Path.DATABASE) as conn:
                                     conn.cursor().execute(
@@ -123,6 +126,7 @@ class Rule:
                                         (self.domain, policy, None, USER_RULE_PRIORITY)
                                     )
                                     conn.commit()
+                                self.__init__(self.domain)
                         if not quiet:
                             print(f" {Icon.SUCCESS} {__msg}")
                     else:
@@ -142,7 +146,8 @@ class Rule:
         :param quiet: Optional. Do not display an output (false by default)
         """
         if not self.exists or not self.is_user_rule:
-            print(f" {Icon.WARNING} No user rule exists for domain: {self.domain}")
+            if not quiet:
+                print(f" {Icon.WARNING} No user rule exists for domain: {self.domain}")
         else:
             __msg = "Deleting rule for domain: {0}".format(self.domain)
             if not quiet:
@@ -153,6 +158,7 @@ class Rule:
                     (self.domain,)
                 )
                 conn.commit()
+            self.__init__(self.domain)
             if not quiet:
                 print(f" {Icon.SUCCESS} {__msg}")
 
@@ -168,7 +174,7 @@ def __remove_allowed_matches(quiet: bool = False) -> dict:
 
     cnt = multiprocessing.Value('i', 0)
 
-    with multiprocessing.Pool(initializer=init_globals, initargs=(cnt,)) as pool:
+    with multiprocessing.Pool(initializer=init_globals, initargs=(cnt, quiet)) as pool:
         wildcards_matches = pool.map(check_wildcard_domains, wildcards)
     if not quiet:
         print(" {0} Checking wildcards rules: {1}/{2}".format(Icon.SUCCESS, cnt.value, len(wildcards)))
@@ -178,15 +184,18 @@ def __remove_allowed_matches(quiet: bool = False) -> dict:
     return matches
 
 
-def init_globals(counter):
+def init_globals(counter, quiet):
     global cnt
     cnt = counter
+    global quiet_mode
+    quiet_mode = quiet
 
 
 def check_wildcard_domains(i) -> dict:
     matches = {}
     cnt.value += 1
-    print(" {0} Checking wildcards rules: {1}".format(loading_icon(cnt.value), cnt.value), end="\r")
+    if not quiet_mode:
+        print(" {0} Checking wildcards rules: {1}".format(loading_icon(cnt.value), cnt.value), end="\r")
     conn = sqlite3.connect(Path.DATABASE)
     cursor = conn.cursor()
     query = cursor.execute("SELECT domain, ip FROM rules WHERE domain LIKE ? AND policy != ?;",
@@ -199,7 +208,7 @@ def check_wildcard_domains(i) -> dict:
 
 
 def allow_domains(domains: list, do_not_prompt: bool = False, force: bool = False, quiet: bool = False,
-                  also_update_hosts: bool = True) -> None:
+                  also_update_hosts: bool = False) -> None:
     """
     Add allowing user rules for a provided list of domains
     :param domains: The list of domains to allow
@@ -236,14 +245,14 @@ def allow_domains(domains: list, do_not_prompt: bool = False, force: bool = Fals
         if also_update_hosts:
             update_hosts(quiet=quiet)
         else:
-            remove_from_hosts({**domains_list, **matches})
+            remove_from_hosts({**domains_list, **matches}, quiet=quiet)
 
     # Unlock the database
     unlock_db()
 
 
 def block_domains(domains: list, do_not_prompt: bool = False, force: bool = False, quiet: bool = False,
-                  also_update_hosts: bool = True) -> None:
+                  also_update_hosts: bool = False) -> None:
     """
     Add blocking user rules for a provided list of domains
     :param domains: The list of domains to block
@@ -265,20 +274,23 @@ def block_domains(domains: list, do_not_prompt: bool = False, force: bool = Fals
         # Add rules
         if not quiet:
             print(f"{Font.BOLD}==> Adding rules{Font.DEFAULT}")
+        domains_to_add = []
         for d in domains:
             rule_object = Rule(d)
             rule_object.add(RulePolicy.BLOCK, quiet=quiet)
+            if rule_object.exists:
+                domains_to_add.append(d)
         if also_update_hosts:
             update_hosts(quiet=quiet)
         else:
-            add_to_hosts(domains)
+            add_to_hosts(domains_to_add, quiet=quiet)
 
     # Unlock the database
     unlock_db()
 
 
 def redirect_domains(domains: list, ip: str, do_not_prompt: bool = False, force: bool = False, quiet: bool = False,
-                     also_update_hosts: bool = True) -> None:
+                     also_update_hosts: bool = False) -> None:
     """
     Add redirecting user rules for a provided list of domains
     :param domains: The list of domains to redirect
@@ -301,20 +313,23 @@ def redirect_domains(domains: list, ip: str, do_not_prompt: bool = False, force:
         # Add rules
         if not quiet:
             print(f"{Font.BOLD}==> Adding rules{Font.DEFAULT}")
+        domains_to_add = []
         for d in domains:
             rule_object = Rule(d)
             rule_object.add(RulePolicy.REDIRECT, ip=ip, quiet=quiet)
+            if rule_object.exists:
+                domains_to_add.append(d)
         if also_update_hosts:
             update_hosts(quiet=quiet)
         else:
-            add_to_hosts(domains, ip)
+            add_to_hosts(domains_to_add, ip, quiet=quiet)
 
     # Unlock the database
     unlock_db()
 
 
 def delete_rules(domains: list, do_not_prompt: bool = False, quiet: bool = False,
-                 also_update_hosts: bool = True) -> None:
+                 also_update_hosts: bool = False) -> None:
     """
     Delete user rules for a provided list of domains
     :param domains: The list of domains to block
@@ -343,12 +358,47 @@ def delete_rules(domains: list, do_not_prompt: bool = False, quiet: bool = False
         if also_update_hosts:
             update_hosts(quiet=quiet)
         else:
-            remove_from_hosts(domains_list)
+            remove_from_hosts(domains_list, quiet=quiet)
 
     # Unlock the database
     unlock_db()
 
 
+def get_all_rules(from_filter_lists: list = None, user_only: bool = False, standard_only: bool = False) -> list:
+    if from_filter_lists and user_only or user_only and standard_only:
+        return []
+    elif not os.path.isfile(Path.DATABASE):
+        raise FileNotFoundError("database does not exist yet. "
+                                "Please run 'tblock -Y' with admin privileges to create it")
+    else:
+        output = []
+        with sqlite3.connect(Path.DATABASE) as conn:
+            if from_filter_lists:
+                for f in from_filter_lists:
+                    data = conn.cursor().execute(
+                        'SELECT "domain" FROM "rules" WHERE filter_id=? ORDER BY domain ASC;',
+                        (f,)
+                    )
+            elif standard_only:
+                data = conn.cursor().execute(
+                    'SELECT "domain" FROM "rules" WHERE filter_id!=? ORDER BY domain ASC;',
+                    (USER_RULE_PRIORITY,)
+                )
+            elif user_only:
+                data = conn.cursor().execute(
+                    'SELECT "domain" FROM "rules" WHERE filter_id=? ORDER BY domain ASC;',
+                    (USER_RULE_PRIORITY,)
+                )
+            else:
+                data = conn.cursor().execute(
+                    'SELECT "domain" FROM "rules" ORDER BY domain ASC;'
+                )
+
+        for rule in data:
+            output.append(rule[0])
+        return output
+
+
 def list_rules(from_filter_lists: list = None, user_only: bool = False, standard_only: bool = False) -> None:
     """
     List rules stored in the database
@@ -368,22 +418,22 @@ def list_rules(from_filter_lists: list = None, user_only: bool = False, standard
             if from_filter_lists:
                 for f in from_filter_lists:
                     data = conn.cursor().execute(
-                        'SELECT "domain", "policy", "ip" FROM "rules" WHERE filter_id=?;',
+                        'SELECT "domain", "policy", "ip" FROM "rules" WHERE filter_id=? ORDER BY domain ASC;',
                         (f,)
                     )
             elif standard_only:
                 data = conn.cursor().execute(
-                    'SELECT "domain", "policy", "ip" FROM "rules" WHERE filter_id!=?;',
+                    'SELECT "domain", "policy", "ip" FROM "rules" WHERE filter_id!=? ORDER BY domain ASC;',
                     (USER_RULE_PRIORITY,)
                 )
             elif user_only:
                 data = conn.cursor().execute(
-                    'SELECT "domain", "policy", "ip" FROM "rules" WHERE filter_id=?;',
+                    'SELECT "domain", "policy", "ip" FROM "rules" WHERE filter_id=? ORDER BY domain ASC;',
                     (USER_RULE_PRIORITY,)
                 )
             else:
                 data = conn.cursor().execute(
-                    'SELECT "domain", "policy", "ip" FROM "rules";'
+                    'SELECT "domain", "policy", "ip" FROM "rules" ORDER BY domain ASC;'
                 )
 
         for rule in data:
@@ -393,3 +443,8 @@ def list_rules(from_filter_lists: list = None, user_only: bool = False, standard
                 print("BLOCK    " + rule[0])
             elif rule[1] == RulePolicy.REDIRECT:
                 print("REDIRECT " + rule[0] + " -> " + rule[2])
+
+
+def get_rules_count() -> int:
+    with sqlite3.connect(Path.DATABASE) as db:
+        return db.cursor().execute('SELECT COUNT() FROM rules;').fetchone()[0]
